/*
 * Copyright (C) 2008-2009 Alex Lasnier <alex@fepg.org>
 *
 * This file is part of NAScan
 *
 * NAScan is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * NAScan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NAScan.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ctype.h>

#include <libsi/section.h>
#include <libsi/descriptor.h>

#include "nitfilter.h"
#include "tools.h"


#define NIT_TID  0x40
#define NIT_PID  0x0010  // NA providers always use Nit pid 16 ?


//////////////////////////////////////////////////////////////////////////////


NitFilter::NitFilter(cList<TransponderSorter>* tl)
{
  transponderList = tl;
  gotNit = false;
  gotData = false;
  sections = 0;
  numSections = 0;
  
  Set(NIT_PID, NIT_TID);
}


//----------------------------------------------------------------------------

NitFilter::~NitFilter()
{

}


//----------------------------------------------------------------------------

void NitFilter::SetStatus(bool On)
{ 
  if (On)
  {
    gotNit = false;
    gotData = false;
    sections = 0;
    numSections = 0;
  }
  
  cFilter::SetStatus(On);  
}


//----------------------------------------------------------------------------

void NitFilter::Process(u_short pid, u_char tid, const u_char* data, int length)
{
  SI::NIT nit(data, false);
  if (!nit.CheckCRCAndParse())
    return;
  
  if ( GotSection(nit.getSectionNumber()) )
    return;
  
  gotData = true;
     
  dprint(L_NIT, "NIT: Received NIT (nid %d) %d of %d.", nit.getNetworkId(), nit.getSectionNumber()+1, nit.getLastSectionNumber()+1);
  
  if (!numSections)
    numSections = nit.getLastSectionNumber() + 1;
  
  SeenSection(nit.getSectionNumber());
  
  SI::NIT::TransportStream ts;
  for (SI::Loop::Iterator it; nit.transportStreamLoop.getNext(ts, it); ) 
  {
    SI::Descriptor *d = NULL;
    for (SI::Loop::Iterator it2; (d = ts.transportStreamDescriptors.getNext(it2)); ) 
    {
      switch (d->getDescriptorTag()) 
      {
        case SI::ServiceListDescriptorTag:
        /*{
          SI::ServiceListDescriptor* sld = (SI::ServiceListDescriptor*) d;
          SI::ServiceListDescriptor::Service ser;
          for (SI::Loop::Iterator it3; sld->serviceLoop.getNext(ser, it3); ) {
            dprint(L_NIT, "NIT: SLD sid %d, type 0x%02X", ser.getServiceId(), ser.getServiceType());
          }
        }*/
        break;
        
        case SI::SatelliteDeliverySystemDescriptorTag: 
        {
          SI::SatelliteDeliverySystemDescriptor* sd = (SI::SatelliteDeliverySystemDescriptor*) d;

          int Frequency = BCD2INT(sd->getFrequency()) / 100;
          char Polarization = sd->getPolarization() == 2 ? 'H' : 'V'; // NA Specific
          
          //dprint(L_NIT, "NIT: %d MHz", Frequency);
          
          // Sometimes sd->getWestEastFlag() is wrong ?!
          int Source = cSource::FromData(cSource::stSat, BCD2INT(sd->getOrbitalPosition()) );//, sd->getWestEastFlag());
          int SymbolRate = BCD2INT(sd->getSymbolRate()) / 10; 
          
          //dprint(L_NIT | L_MSG, "NIT: Found transponder (%s %d:%c)", *cSource::ToString(Source), Frequency, toupper(Polarization));
          
          // Values only valid for VDR 1.7.0 ?
          //static int CodeRates[]   = { 0, 12, 23, 34, 56, 78, 89, 35, 45, 910, 999, 999, 999, 999, 999, 0 };
                                  // { DVBFE_MOD_AUTO, DVBFE_MOD_QPSK, DVBFE_MOD_8PSK, DVBFE_MOD_QAM16 };
          //static int Modulations[] = {            999,              2,              5,              16 };
            
          // int CodeRate   = MapToDriver(CodeRates[sd->getFecInner()], CoderateValues);
          int CodeRate   = MapToDriver(999, CoderateValues); // Auto
          //int Modulation = MapToDriver(Modulations[sd->getModulationType()], ModulationValues);
          int Modulation = MapToDriver(999, ModulationValues);
          
#if VDRVERSNUM < 10700
          int System   = 0;
          int RollOff  = 0;
#else           
          int System   = MapToDriver(sd->getModulationSystem() ? 1 : 0, SystemValues);
          
          static int RollOffs[] = { 35, 25, 20, 0 };
          int RollOff  = MapToDriver(sd->getModulationSystem() ? RollOffs[sd->getRollOff()] : 0, RollOffValues);
#endif            
          //dprint(L_NIT, "NIT: cr %d, mod %d, sys %d, ro %d", CodeRate, Modulation, System, RollOff); 
          
          AddTransponder(Source, ts.getOriginalNetworkId(), ts.getTransportStreamId(), Frequency, Polarization, SymbolRate, CodeRate, Modulation, System, RollOff);                  
        }
        break;

        default:
          dprint(L_NIT, "NIT: Unsupported Delivery System Descriptor 0x%02X", d->getDescriptorTag()); 
        break;
      }
      
      delete d;
    }
  }
  
  
  if (GotAllSections()) {
    dprint(L_NIT, "NIT: Got all sections");
    
    gotNit = true;
    SetStatus(false);
    
    /*
    transponderList->Sort();
    for(TransponderSorter* ts = transponderList->First(); ts; ts = transponderList->Next(ts))
      dprint(L_NIT | L_MSG, "%d:%c", ts->channel->Frequency(), toupper(ts->channel->Polarization()));
    */
  }
}


//----------------------------------------------------------------------------

bool NitFilter::GotSection(int i) const
{
  if (i < 0 || i >= numSections)
    return false;

  return (sections >> i) & 1;
}


//----------------------------------------------------------------------------  

bool NitFilter::GotAllSections(void) const
{
  uint64_t i = 1;
  i = i << numSections;

  return (sections == i-1);
}


//----------------------------------------------------------------------------  

void NitFilter::SeenSection(int i)
{
  if (i < 0 || i >= numSections)
    return;
    
  sections |= (1 << i);
}


//---------------------------------------------------------------------------- 

void NitFilter::AddTransponder(int Source, int Nid, int Tid, int Frequency, char Polarization, int Srate, int CoderateH, int Modulation, int System, int RollOff)
{
  for(TransponderSorter* ts = transponderList->First(); ts; ts = transponderList->Next(ts))
    if (ts->channel->Frequency() ==  Frequency)
      return;
   
  cChannel* curTransponder = new cChannel();
  
#if VDRVERSNUM < 10700
  curTransponder->SetSatTransponderData(Source, Frequency, Polarization, Srate, CoderateH);
#else
  curTransponder->SetSatTransponderData(Source, Frequency, Polarization, Srate, CoderateH, Modulation, System, RollOff);
#endif
  
  curTransponder->SetId(Nid, Tid, 0, 0);
  
  transponderList->Add(new TransponderSorter(curTransponder));
}


//////////////////////////////////////////////////////////////////////////////


