/*
 * Copyright (C) 2008-2009 Alex Lasnier <alex@fepg.org>
 *
 * This file is part of NAScan
 *
 * NAScan is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * NAScan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NAScan.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <libsi/section.h>
#include <libsi/descriptor.h>

#include "sdtfilter.h"
#include "tools.h"


#define SDT_KEY(NID,TSID,SID) ((((uint64_t) NID) << 32) | (((uint64_t) TSID) << 16) | ((uint64_t) SID))

#define SDT_PID    0x0011
#define SDT_TID    0x42  // Actual transport stream
#define SDT_TID_O  0x46  // Other transport stream 


//////////////////////////////////////////////////////////////////////////////


SdtFilter::SdtFilter(void)
{
  gotAllSdts = gotActualSdt = gotSdtData = false;
  source = 0;
  
  Set(SDT_PID, SDT_TID);
  Set(SDT_PID, SDT_TID_O);
}


//----------------------------------------------------------------------------

SdtFilter::~SdtFilter(void)
{

}


//----------------------------------------------------------------------------

const char* SdtFilter::GetName(uint16_t nid, uint16_t tid, uint16_t sid) const
{
  std::map<uint64_t, std::string>::const_iterator itr = channelNames.find(SDT_KEY(nid, tid, sid));
  if (itr != channelNames.end())
    return (*itr).second.c_str();
  else {
    dprint(L_SDT, "No name for %d %d %d", nid, tid, sid);
    return NULL;   
  }
}


//----------------------------------------------------------------------------

void SdtFilter::SetStatus(bool On)
{
  if (On && gotAllSdts && source == Source())
  {
    dprint(L_DBG, "SDT Filter: same source. No reset.");
  }
  else if (On)
  {
    dprint(L_DBG, "SDT Filter: diffrent source. Clearing SDT cache.");
    gotAllSdts = gotActualSdt = false;
    gotSdtData = false;
    channelNames.clear();
    source = Source();
  }
  
  cFilter::SetStatus(On);
}


//----------------------------------------------------------------------------

void SdtFilter::Process(u_short pid, u_char tid, const u_char* data, int length)
{ 
  if (tid == SDT_TID) { // Actual TS SDT may cause premature duplicate detection
    if (gotActualSdt) {
      dprint(L_DBG, "SDT: Actual TS SDT already seen."); 
      return;
    }  
    gotActualSdt = true;
  }
  
  SI::SDT sdt(data, false);
  if (!sdt.CheckCRCAndParse())
    return;
  
  gotSdtData = true;
 
  SI::SDT::Service SiSdtService;
  for (SI::Loop::Iterator it; sdt.serviceLoop.getNext(SiSdtService, it); )
  {
    uint64_t key = SDT_KEY(sdt.getOriginalNetworkId(), sdt.getTransportStreamId(), SiSdtService.getServiceId());
    
    std::map<uint64_t, std::string>::const_iterator itr = channelNames.find(key);
    if (itr != channelNames.end()) { // Key found
      // When we start getting duplicates, it's probably safe to stop.
      dprint(L_SDT|L_DBG, "SDT: Received duplicate SDT (%d %d %d)", sdt.getOriginalNetworkId(), sdt.getTransportStreamId(), SiSdtService.getServiceId());
      gotAllSdts = true;
      SetStatus(false);
      return;
    }
    
    SI::Descriptor* d;
    for (SI::Loop::Iterator it2; (d = SiSdtService.serviceDescriptors.getNext(it2)); ) 
    {
      switch (d->getDescriptorTag()) 
      {
        case SI::ServiceDescriptorTag: // 0x48
        {
          SI::ServiceDescriptor *sd = (SI::ServiceDescriptor*) d;
          dprint(L_SDT, "SDT: Service Descriptor type: 0x%02X", sd->getServiceType());
          
          char NameBuf[sd->serviceName.getLength()+1];
          sd->serviceName.getText(NameBuf, sizeof(NameBuf));
          dprint(L_SDT, "     %s", NameBuf);
          
          if (!isempty(NameBuf))
            channelNames[key] = NameBuf;
          else
            channelNames[key] = "No Name";

// TODO: Ignore certain channels based on service type.
#if 0          
          switch (sd->getServiceType())
          {
            case 0x01: // digital television service
            case 0x02: // digital radio sound service

            case 0x86: // Interactive ?
            case 0x89:
            case 0x96:
            case 0x9A:
            case 0xA0:
            case 0xA1:
            case 0xA4:
            case 0xA8:
            {
              char NameBuf[sd->serviceName.getLength()+1];
              sd->serviceName.getText(NameBuf, sizeof(NameBuf));
              dprint(L_SDT, "     %s", NameBuf);
              
              /* No interesting info for NA providers
              char ProviderNameBuf[sd->providerName.getLength()+1];
              sd->providerName.getText(ProviderNameBuf, sizeof(ProviderNameBuf));
              dprint(L_SDT, "     %s", ProviderNameBuf);
              */
              
            }
            break;
            
            case 0x8D: // EPG
            case 0x8F: // ???
            case 0x90: 
            default:
              break;
          }
#endif          
        }
        break;
        
        case SI::LinkageDescriptorTag: // 0x4A: EPG Link
          dprint(L_SDT, "SDT: Linkage Descriptor");
        break;

        case 0x80: // Dish channel mapping
        case 0x82: // EPG channel
        case 0x84: // ???  
        case 0x85: // Password/Region protection     
        case 0x86: // Tier descriptor     
        case 0x93: // Tier rights descriptor
        case 0x9E: // Dish channel available
        case 0x99: // ATSC ?
          dprint(L_SDT, "SDT: Unsupported tag: 0x%02X", d->getDescriptorTag());
        break;
        
        default:
          dprint(L_SDT, "SDT: Unknown tag: 0x%02X", d->getDescriptorTag());
        break;
      }
      
      delete d;
      
    }
  }
}


//////////////////////////////////////////////////////////////////////////////


